@@ -9,9 +9,10 @@ from pykusto.client import Table, KustoResponse
 from pykusto.expressions import BooleanType, ExpressionType, AggregationExpression, OrderedType, \
     StringType, AssignmentBase, AssignmentFromAggregationToColumn, AssignmentToSingleColumn, AnyTypeColumn, \
     BaseExpression, \
-    AssignmentFromColumnToColumn, AnyExpression, to_kql, ColumnToType
+    AssignmentFromColumnToColumn, AnyExpression, to_kql, expression_to_type, BaseColumn
+from pykusto.kql_converters import KQL
 from pykusto.logger import logger
-from pykusto.type_utils import KustoType, KQL
+from pykusto.type_utils import KustoType, typed_column, plain_expression
 from pykusto.udf import stringify_python_func
 
 
@@ -104,22 +105,8 @@ class Query:
     def join(self, query: 'Query', kind: JoinKind = None) -> 'JoinQuery':
         return JoinQuery(self, query, kind)
 
-    def project(
-            self, *args: Union[AnyTypeColumn, AssignmentBase, BaseExpression], **kwargs: ExpressionType
-    ) -> 'ProjectQuery':
-        columns: List[AnyTypeColumn] = []
-        assignments: List[AssignmentBase] = []
-        for arg in args:
-            if isinstance(arg, AnyTypeColumn):
-                columns.append(arg)
-            elif isinstance(arg, AssignmentBase):
-                assignments.append(arg)
-            else:
-                assert isinstance(arg, BaseExpression), "Invalid assignment"
-                assignments.append(arg.assign_to())
-        for column_name, expression in kwargs.items():
-            assignments.append(AssignmentToSingleColumn(AnyTypeColumn(column_name), expression))
-        return ProjectQuery(self, columns, assignments)
+    def project(self, *args: Union[AssignmentBase, BaseExpression], **kwargs: ExpressionType) -> 'ProjectQuery':
+        return ProjectQuery(self, self.extract_assignments(*args, **kwargs))
 
     def project_rename(self, *args: AssignmentFromColumnToColumn, **kwargs: AnyTypeColumn) -> 'ProjectRenameQuery':
         assignments: List[AssignmentFromColumnToColumn] = list(args)
@@ -137,18 +124,7 @@ class Query:
         return DistinctQuery(self, (AnyExpression(KQL("*")),))
 
     def extend(self, *args: Union[BaseExpression, AssignmentBase], **kwargs: ExpressionType) -> 'ExtendQuery':
-        assignments: List[AssignmentBase] = []
-        for arg in args:
-            if isinstance(arg, BaseExpression):
-                assignments.append(arg.assign_to())
-            else:
-                assignments.append(arg)
-        for column_name, expression in kwargs.items():
-            if isinstance(expression, BaseExpression):
-                assignments.append(expression.assign_to(AnyTypeColumn(column_name)))
-            else:
-                assignments.append(AnyExpression(to_kql(expression)).assign_to(AnyTypeColumn(column_name)))
-        return ExtendQuery(self, *assignments)
+        return ExtendQuery(self, *self.extract_assignments(*args, **kwargs))
 
     def summarize(self, *args: Union[AggregationExpression, AssignmentFromAggregationToColumn],
                   **kwargs: AggregationExpression) -> 'SummarizeQuery':
@@ -164,12 +140,13 @@ class Query:
         return SummarizeQuery(self, assignments)
 
     def mv_expand(
-            self, *columns: Union[AnyTypeColumn, ColumnToType], bag_expansion: BagExpansion = None,
-            with_item_index: AnyTypeColumn = None, limit: int = None
+            self, *args: Union[BaseExpression, AssignmentBase], bag_expansion: BagExpansion = None,
+            with_item_index: BaseColumn = None, limit: int = None, **kwargs: ExpressionType
     ) -> 'MvExpandQuery':
-        if len(columns) == 0:
+        assignments = self.extract_assignments(*args, **kwargs)
+        if len(assignments) == 0:
             raise ValueError("Please specify one or more columns for mv-expand")
-        return MvExpandQuery(self, columns, bag_expansion, with_item_index, limit)
+        return MvExpandQuery(self, bag_expansion, with_item_index, limit, *assignments)
 
     def custom(self, custom_query: str) -> 'CustomQuery':
         return CustomQuery(self, custom_query)
@@ -182,11 +159,7 @@ class Query:
     ) -> 'EvaluateQuery':
         return EvaluateQuery(
             self, 'python',
-            AnyExpression(
-                KQL('typeof({})'.format(('*, ' if extend else '') + ', '.join(
-                    field_name + ':' + kusto_type.primary_name for field_name, kusto_type in type_specs.items()
-                )))
-            ),
+            AnyExpression(KQL(f'typeof({("*, " if extend else "") + ", ".join(field_name + ":" + kusto_type.primary_name for field_name, kusto_type in type_specs.items())})')),
             stringify_python_func(udf),
             distribution=distribution
         )
@@ -211,7 +184,7 @@ class Query:
                 else:
                     return table.to_query_format()
         else:
-            return KQL("{} | {}".format(self._head._compile_all(use_full_table_name), self._compile()))
+            return KQL(f"{self._head._compile_all(use_full_table_name)} | {self._compile()}")
 
     def get_table(self) -> Table:
         if self._head is None:
@@ -241,21 +214,34 @@ class Query:
     def to_dataframe(self, table: Table = None):
         return self.execute(table).to_dataframe()
 
+    @staticmethod
+    def extract_assignments(*args: Union[AssignmentBase, BaseExpression], **kwargs: ExpressionType) -> List[AssignmentBase]:
+        assignments: List[AssignmentBase] = []
+        for arg in args:
+            if isinstance(arg, BaseExpression):
+                assignments.append(arg.assign_to())
+            else:
+                assignments.append(arg)
+        for column_name, expression in kwargs.items():
+            column_type = expression_to_type(expression, typed_column, AnyTypeColumn)
+            if isinstance(expression, BaseExpression):
+                assignments.append(expression.assign_to(column_type(column_name)))
+            else:
+                expression_type = expression_to_type(expression, plain_expression, AnyExpression)
+                assignments.append(expression_type(to_kql(expression)).assign_to(column_type(column_name)))
+        return assignments
+
 
 class ProjectQuery(Query):
     _columns: List[AnyTypeColumn]
     _assignments: List[AssignmentBase]
 
-    def __init__(self, head: 'Query', columns: List[AnyTypeColumn], assignments: List[AssignmentBase]) -> None:
+    def __init__(self, head: 'Query', assignments: List[AssignmentBase]) -> None:
         super().__init__(head)
-        self._columns = columns
         self._assignments = assignments
 
     def _compile(self) -> KQL:
-        return KQL('project {}'.format(', '.join(chain(
-            (c.kql for c in self._columns),
-            (a.to_kql() for a in self._assignments)
-        ))))
+        return KQL(f"project {', '.join(a.to_kql() for a in self._assignments)}")
 
 
 class ProjectRenameQuery(Query):
@@ -266,7 +252,7 @@ class ProjectRenameQuery(Query):
         self._assignments = assignments
 
     def _compile(self) -> KQL:
-        return KQL('project-rename {}'.format(', '.join(a.to_kql() for a in self._assignments)))
+        return KQL(f"project-rename {', '.join(a.to_kql() for a in self._assignments)}")
 
 
 class ProjectAwayQuery(Query):
@@ -277,7 +263,7 @@ class ProjectAwayQuery(Query):
         self._columns = columns
 
     def _compile(self) -> KQL:
-        return KQL('project-away {}'.format(', '.join((str(c) for c in self._columns))))
+        return KQL(f"project-away {', '.join(str(c) for c in self._columns)}")
 
 
 class DistinctQuery(Query):
@@ -288,7 +274,7 @@ class DistinctQuery(Query):
         self._columns = columns
 
     def _compile(self) -> KQL:
-        return KQL('distinct {}'.format(', '.join((c.kql for c in self._columns))))
+        return KQL(f"distinct {', '.join(c.kql for c in self._columns)}")
 
 
 class ExtendQuery(Query):
@@ -299,7 +285,7 @@ class ExtendQuery(Query):
         self._assignments = assignments
 
     def _compile(self) -> KQL:
-        return KQL('extend {}'.format(', '.join(a.to_kql() for a in self._assignments)))
+        return KQL(f"extend {', '.join(a.to_kql() for a in self._assignments)}")
 
 
 class WhereQuery(Query):
@@ -310,7 +296,7 @@ class WhereQuery(Query):
         self._predicate = predicate
 
     def _compile(self) -> KQL:
-        return KQL('where {}'.format(self._predicate.kql))
+        return KQL(f'where {self._predicate.kql}')
 
 
 class _SingleNumberQuery(Query):
@@ -323,7 +309,7 @@ class _SingleNumberQuery(Query):
         self._num_rows = num_rows
 
     def _compile(self) -> KQL:
-        return KQL('{} {}'.format(self._query_name, self._num_rows))
+        return KQL(f'{self._query_name} {self._num_rows}')
 
 
 class TakeQuery(_SingleNumberQuery):
@@ -391,9 +377,7 @@ class _OrderQueryBase(Query):
         return res
 
     def _compile(self) -> KQL:
-        return KQL(
-            '{} by {}'.format(self._query_name,
-                              ", ".join([self._compile_order_spec(order_spec) for order_spec in self._order_specs])))
+        return KQL(f'{self._query_name} by {", ".join([self._compile_order_spec(order_spec) for order_spec in self._order_specs])}')
 
 
 class SortQuery(_OrderQueryBase):
@@ -417,7 +401,7 @@ class TopQuery(Query):
 
     def _compile(self) -> KQL:
         # noinspection PyProtectedMember
-        return KQL('top {} by {}'.format(self._num_rows, SortQuery._compile_order_spec(self._order_spec)))
+        return KQL(f'top {self._num_rows} by {SortQuery._compile_order_spec(self._order_spec)}')
 
 
 class JoinException(Exception):
@@ -446,7 +430,7 @@ class JoinQuery(Query):
         if len(attribute) == 1:
             return attribute[0].kql
         else:
-            return "$left.{}==$right.{}".format(attribute[0].kql, attribute[1].kql)
+            return f"$left.{attribute[0].kql}==$right.{attribute[1].kql}"
 
     def _compile(self) -> KQL:
         if len(self._on_attributes) == 0:
@@ -454,10 +438,9 @@ class JoinQuery(Query):
         if self._joined_query.get_table() is None:
             raise JoinException("The joined query must have a table")
 
-        return KQL("join {} ({}) on {}".format(
-            "" if self._kind is None else "kind={}".format(self._kind.value),
-            self._joined_query.render(use_full_table_name=True),
-            ", ".join([self._compile_on_attribute(attr) for attr in self._on_attributes])))
+        return KQL(f'join {"" if self._kind is None else f"kind={self._kind.value}"} '
+                   f'({self._joined_query.render(use_full_table_name=True)}) on '
+                   f'{", ".join([self._compile_on_attribute(attr) for attr in self._on_attributes])}')
 
 
 class SummarizeQuery(Query):
@@ -485,27 +468,21 @@ class SummarizeQuery(Query):
         return self
 
     def _compile(self) -> KQL:
-        result = 'summarize {}'.format(', '.join(a.to_kql() for a in self._assignments))
+        result = f"summarize {', '.join(a.to_kql() for a in self._assignments)}"
         if len(self._by_assignments) != 0 or len(self._by_columns) != 0:
-            result += ' by {}'.format(', '.join(chain(
-                (c.kql for c in self._by_columns),
-                (a.to_kql() for a in self._by_assignments)
-            )))
+            result += f' by {", ".join(chain((c.kql for c in self._by_columns), (a.to_kql() for a in self._by_assignments)))}'
         return KQL(result)
 
 
 class MvExpandQuery(Query):
-    _columns: Tuple[Union[AnyTypeColumn, ColumnToType]]
+    _assignments: Tuple[AssignmentBase]
     _bag_expansion: BagExpansion
-    _with_item_index: AnyTypeColumn
+    _with_item_index: BaseColumn
     _limit: int
 
-    def __init__(
-            self, head: Query, columns: Tuple[Union[AnyTypeColumn, ColumnToType]], bag_expansion: BagExpansion,
-            with_item_index: AnyTypeColumn, limit: int
-    ):
+    def __init__(self, head: Query, bag_expansion: BagExpansion, with_item_index: BaseColumn, limit: int, *assignments: AssignmentBase):
         super(MvExpandQuery, self).__init__(head)
-        self._columns = columns
+        self._assignments = assignments
         self._bag_expansion = bag_expansion
         self._with_item_index = with_item_index
         self._limit = limit
@@ -513,12 +490,12 @@ class MvExpandQuery(Query):
     def _compile(self) -> KQL:
         res = "mv-expand "
         if self._bag_expansion is not None:
-            res += "bagexpansion={} ".format(self._bag_expansion.value)
+            res += f"bagexpansion={self._bag_expansion.value} "
         if self._with_item_index is not None:
-            res += "with_itemindex={} ".format(self._with_item_index.kql)
-        res += ", ".join([c.kql for c in self._columns])
+            res += f"with_itemindex={self._with_item_index.kql} "
+        res += ", ".join(a.to_kql() for a in self._assignments)
         if self._limit:
-            res += " limit {}".format(self._limit)
+            res += f" limit {self._limit}"
         return KQL(res)
 
 
@@ -545,8 +522,5 @@ class EvaluateQuery(Query):
         self._distribution = distribution
 
     def _compile(self) -> KQL:
-        return KQL('evaluate {}{}({})'.format(
-            '' if self._distribution is None else 'hint.distribution={} '.format(self._distribution.value),
-            self._plugin_name,
-            ', '.join(to_kql(arg) for arg in self._args),
-        ))
+        return KQL(f'evaluate {"" if self._distribution is None else f"hint.distribution={self._distribution.value} "}'
+                   f'{self._plugin_name}({", ".join(to_kql(arg) for arg in self._args)})')
