@@ -460,12 +460,6 @@ class TestFunction(TestBase):
             Query().where(f.to_bool(t.stringField)).render()
         )
 
-    def test_toboolean(self):
-        self.assertEqual(
-            " | where toboolean(stringField)",
-            Query().where(f.to_boolean(t.stringField)).render()
-        )
-
     def test_todouble(self):
         self.assertEqual(
             " | where (todouble(stringField)) > 0.2",
@@ -801,6 +795,20 @@ class TestFunction(TestBase):
             Query().project(foo=f.iff(t.dateField > f.ago(timedelta(2)), 2, f.array_length(t.arrayField))).render()
         )
 
+    def test_iff_ambiguous_type(self):
+        with self.assertLogs(logger, logging.WARN) as cm:
+            # We want to assert there are no warnings, but the 'assertLogs' method does not support that.
+            # Therefore, we are adding a dummy warning, and then we will assert it is the only warning.
+            logger.warn("Dummy warning")
+            self.assertEqual(
+                " | project foo = iff(boolField, time(3.0:0:0.0), foo - bar)",
+                Query().project(foo=f.iff(t.boolField, timedelta(3), col.foo - col.bar)).render()
+            )
+        self.assertEqual(
+            ["WARNING:pykusto:Dummy warning"],
+            cm.output,
+        )
+
     def test_iif(self):
         # iif is just an alias to iff
         self.assertEqual(
@@ -819,3 +827,81 @@ class TestFunction(TestBase):
             ' | extend foo = pack_array(numField, stringField)',
             Query().extend(foo=f.pack_array(t.numField, t.stringField)).render()
         )
+
+    def test_set_has_element(self):
+        self.assertEqual(
+            ' | extend foo = set_has_element(arrayField, numField)',
+            Query().extend(foo=f.set_has_element(t.arrayField, t.numField)).render()
+        )
+
+    def test_set_difference(self):
+        self.assertEqual(
+            ' | extend foo = set_difference(arrayField, arrayField2)',
+            Query().extend(foo=f.set_difference(t.arrayField, t.arrayField2)).render()
+        )
+
+    def test_set_intersect(self):
+        self.assertEqual(
+            ' | extend foo = set_intersect(arrayField, arrayField2)',
+            Query().extend(foo=f.set_intersect(t.arrayField, t.arrayField2)).render()
+        )
+
+    def test_set_union(self):
+        self.assertEqual(
+            ' | extend foo = set_union(arrayField, arrayField2)',
+            Query().extend(foo=f.set_union(t.arrayField, t.arrayField2)).render()
+        )
+
+    def test_array_concat(self):
+        self.assertEqual(
+            ' | extend foo = array_concat(arrayField, arrayField2)',
+            Query().extend(foo=f.array_concat(t.arrayField, t.arrayField2)).render()
+        )
+
+    def test_array_iif(self):
+        self.assertEqual(
+            ' | extend foo = array_iif(arrayField, arrayField2, arrayField3)',
+            Query().extend(foo=f.array_iif(t.arrayField, t.arrayField2, t.arrayField3)).render()
+        )
+
+    def test_array_index_of(self):
+        self.assertEqual(
+            ' | extend foo = array_index_of(arrayField, numField)',
+            Query().extend(foo=f.array_index_of(t.arrayField, t.numField)).render()
+        )
+
+    def test_array_rotate_left(self):
+        self.assertEqual(
+            ' | extend foo = array_rotate_left(arrayField, numField)',
+            Query().extend(foo=f.array_rotate_left(t.arrayField, t.numField)).render()
+        )
+
+    def test_array_rotate_right(self):
+        self.assertEqual(
+            ' | extend foo = array_rotate_right(arrayField, numField)',
+            Query().extend(foo=f.array_rotate_right(t.arrayField, t.numField)).render()
+        )
+
+    def test_array_shift_left(self):
+        self.assertEqual(
+            ' | extend foo = array_shift_left(arrayField, numField)',
+            Query().extend(foo=f.array_shift_left(t.arrayField, t.numField)).render()
+        )
+
+    def test_array_shift_right(self):
+        self.assertEqual(
+            ' | extend foo = array_shift_right(arrayField, numField, numField2)',
+            Query().extend(foo=f.array_shift_right(t.arrayField, t.numField, t.numField2)).render()
+        )
+
+    def test_array_slice(self):
+        self.assertEqual(
+            ' | extend foo = array_slice(arrayField, numField, numField2)',
+            Query().extend(foo=f.array_slice(t.arrayField, t.numField, t.numField2)).render()
+        )
+
+    def test_array_split(self):
+        self.assertEqual(
+            ' | extend foo = array_split(arrayField, numField)',
+            Query().extend(foo=f.array_split(t.arrayField, t.numField)).render()
+        )
